## The Project Name
PROJECTNAME = "GetPaidSalesforceOrderRecorder"

## The skins dir
SKINS_DIR = 'skins'

## Globals variable
GLOBALS = globals()

## Required field marker
REQUIRED_MARKER = "(required)"

